#!/usr/bin/perl -w
# -*- mode: CPerl; tab-width: 8; fill-column: 70; -*- 

use Template;
use File::Temp qw/ tempdir /;
use HTML::TreeBuilder;
use Pod::Usage;
use Getopt::Long;

my $target_dir = "";
my $modules_dir = "";
my $issue_file = "";
my $prefix = "";
my $man = 0;
my $help = 0;
my $template_dir = '/usr/share/docs-build/docs_issue';

GetOptions('help|?'		=> \$help,
	   'm|modules-dir=s'	=> \$modules_dir,
	   'o|output-dir=s'	=> \$target_dir,
	   'i|issue=s'		=> \$issue_file,
	   'p|prefix=s'		=> \$prefix,
	   'man' 		=> \$man)
  or pod2usage( -exitval => 1, -output => \*STDERR );
pod2usage(1) if $help;
pod2usage(-verbose => 2) if $man;
pod2usage( -msg => "Error: Incorrect argument", -exitval => 1, -output => \*STDERR ) if ! -f $issue_file or ! -d $target_dir;

# or ! -d $modules_dir

my $entities = '<>';
my @prev_div = ( "index.html" );

sub mklink {
  my ($f, $d, $new_n) = @_;
  $new_n = "" unless $new_n;
  $f =~ s/^(.*)\/([^\/]*)$/$1/;
  my $n = $2;
  my $r = `relative "$f" "$d/"` || die $!;
  chomp($r);
  symlink("$r/$n", "$d/$new_n");
}

sub mklinkdir {
  my ($f, $d) = @_;
  $d =~ s/^(.*)\/([^\/]*)$/$1/;
  my $n = $2;
  my $r = `relative "$f" "$d/"` || die $!;
  chomp($r);
  symlink("$r", "$d/$n");
}

sub GetTag1 {
  my ($tree, $tag, $type, $class) = @_;
  my $node = "";
  $tree->look_down('_tag', $tag, $type, $class, sub { $node = $_[0] } );

  return ("") if !$node;

  my $t = HTML::Element->new('div');
  foreach my $item ( $node->content_refs_list ) {
    $t->push_content($$item);
  }
  $node->delete;
  return($t);
}
sub GetTag {
  my ($tree, $tag) = @_;
  my $node = "";
  $tree->look_down('_tag', $tag, sub { $node = $_[0] } );

  return ("") if !$node;

  my $t = HTML::Element->new('div');
  foreach my $item ( $node->content_refs_list ) {
    $t->push_content($$item);
  }
  $node->delete;
  return($t);
}

sub dump_file {
    my $template = Template->new({ INCLUDE_PATH => $template_dir });
    $template->process($_[0], $_[2], $_[1]) || die $template->error();
}

sub Up {
  my ($curr_mod, $up_file) = @_;
  mkdir("$target_dir/$curr_mod.up");
  dump_file("up.html.tmpl", "$target_dir/$curr_mod.up/index.html", { up_file => $up_file });
}

sub Nav {
  my ($prev_mod, $curr_mod, $suffix) = @_;
  return unless $curr_mod;

  if (!$prev_mod) {
    mklinkdir "$target_dir/$curr_mod.up", "$target_dir/$curr_mod.$suffix";
  } else {
    mklinkdir "$target_dir/$prev_mod", "$target_dir/$curr_mod.$suffix";
  }
}

sub LinkAHref {
  my $prev_name = "";

  $_[0]->look_down("_tag", "a",
		   sub {
		     my $href = $_[0]->attr('href') || "";
		     return(0) unless "$href" =~ /^adt:/;
#		     $href =~ /^adt:([^.]*)\..*$/;
		     $href =~ /^adt:(.*)$/;
		     my $modname = "$1";
		     $_[0]->attr('href', "$modname/index.html");
		     mklinkdir "$prefix$modules_dir/$modname", "$target_dir/$modname";
		     Up($modname,$prev_div[$#prev_div]);
		     Nav($prev_name, $modname, "prev");
		     Nav($modname, $prev_name, "next");
		     $prev_name = $modname;
		     return(0);
		   });
  Nav("", $prev_name, "next") if $prev_name;
}

sub SubSection {
  foreach my $item ( $_[0]->content_refs_list ) {
    next unless ref $$item;

    my $div_class = $$item->attr('class') || "";
    my $id = $$item->attr('id') || "";

    if ($$item->{_tag} eq "div" && "$div_class" eq "subsection") {
      push(@prev_div, "index_$id.html");
    }

    SubSection($$item);

    if ($$item->{_tag} eq "div" && "$div_class" eq "subsection") {
      event($$item);
      pop(@prev_div);
    }
  }
}

sub event {
  my ($node) = @_;

  my $title = $node->attr('title') || "";
  my $id = $node->attr('id') || "";

  die "attribute 'id' not found" if !$id or !$title;

  my $elem = HTML::Element->new('a', 'href' => "index_$id.html");
  $elem->push_content($title);
  $node->replace_with($elem);

  my $html = HTML::Element->new('html');
  $html->push_content($header);
  $html->look_down("_tag", "title", sub { $_[0]->delete_content(); $_[0]->push_content($title); });

  my $body = HTML::Element->new('body');
  {
    my $nav = HTML::TreeBuilder->new();
    $nav->parse_file("$template_dir/nav.html.tmpl");

    $nav->look_down("_tag", "a", sub {
		      return(0) unless $_[0]->attr('href') =~ /^nav:up/;
		      $_[0]->attr('href', $prev_div[$#prev_div-1]);
		      return(0);
		    });
    $body->push_content($nav);
  }

  foreach my $item ( $node->content_refs_list ) {
    $body->push_content($$item);
  }

  LinkAHref($body);

  $html->push_content($body);
  dump_file("index.html.tmpl", "$target_dir/index_$id.html", { content => $html->as_HTML($entities,'   ',{}) } );
}

###
### Main
###

$temp = tempdir( "tempdir.XXXXXXXXXXX", CLEANUP => 1, TMPDIR => 1 );
open(FH, ">", "$temp/issue_file") || die;
open(FHI, $issue_file) || die;
while(<FHI>) {
  s/&/&amp;/g;
  print FH $_;
}
close(FHI);
close(FH);

my $tree = HTML::TreeBuilder->new();
$tree->parse_file("$temp/issue_file");

$header = "";
$tree->look_down('_tag', 'head', sub { $header = $_[0]->clone } );

SubSection($tree);
LinkAHref($tree);

dump_file("index.html.tmpl", "$target_dir/index.html", { content => $tree->as_HTML($entities,'   ',{}) });

$tree->delete;

__END__

=head1 NAME

docs_issue.pl - catalog hacking program (aka scrollkeeper).

=head1 SYNOPSIS

docs_issue.pl -m <modules_dir> -o <output_dir> -i <issue_file>

=head1 OPTIONS

=over 25

=item B<-m, --modules-dir=DIR>

Real modules dir.

=item B<-o, --output-dir=DIR>

Output dir.

=item B<-i, --issue=FILE>

issue file.

=item B<-?, -help>

Print a brief help message and exits.

=item B<-man>

Prints the manual page and exits.

=back

=head1 DESCRIPTION

docs_mkcatalog.pl is a cataloging system for documentation. It manages documentation metadata.

=cut
