package Heap::Archive;

use Archive::Tar;

my $tar = undef;
my $ARCHIVE = undef;
my $archive_info = {};
my $compress = {
		bzip2 	=> 'bzip2 -q -dc',
		gzip	=> 'gzip -q -dc',
	       };

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;

  $tar = Archive::Tar->new;

  $archive_info->{FULLNAME} = $_[0] if -f $_[0];
  init();

  return $class;
}

sub init {
  return unless defined $archive_info->{FULLNAME};

  $archive_info->{FULLNAME} =~ /\/?([^\/]+)$/;
  $archive_info->{FILENAME} = $1;

  $archive_info->{FILENAME} =~ /^(.*).(bz2?|tbz2?|gz|tgz|z|Z)$/;
  $archive_info->{NAME} = $1;

  $archive_info->{FILENAME} =~ /^.*\.([^.]*)/;
  $archive_info->{SUFFIX} = $1;

  $archive_info->{COMPRESSTYPE} = 'bzip2' if $1 =~ /^(bz2?|tbz2?)$/;
  $archive_info->{COMPRESSTYPE} = 'gzip'  if $1 =~ /^(gz|tgz|z|Z)$/;

}

sub open_archive {
  return if defined $ARCHIVE;
  return unless -f $archive_info->{FULLNAME};
  open($ARCHIVE, "$compress->{$archive_info->{COMPRESSTYPE}} < '$archive_info->{FULLNAME}' |")
    or die "Error: can not open archive\n";
  $tar->read($ARCHIVE,0);
}

sub close_archive {
  return unless defined $ARCHIVE;
  close($ARCHIVE);
  $tar->clear;
  $ARCHIVE = undef;
}

sub get_info {
  return undef unless defined $archive_info->{FULLNAME};
  return $archive_info;
}

sub get_file {
  my $self = shift;
  my ($filename, $options) = @_;

  return unless defined $archive_info->{FULLNAME};
  open_archive();
  $filename =~ /([^\/]+)$/;
  my $destfile = "./$1";

  if ( $tar->contains_file($filename) ) {
    if (defined $options->{DEST_DIR}) {
      #mkdir $options->{DEST_DIR} unless -d $options->{DEST_DIR};
      $filename =~ /([^\/]+)$/;
      $destfile = $options->{DEST_DIR}."/".$1;
    }
    if (defined $options->{DEST_FILE}) {
      $destfile = $options->{DEST_FILE};
    }
    # print "# $filename => $destfile\n";
    $tar->extract_file($filename, $destfile);
  }
  close_archive();
}

sub list($) {
  my $self = shift;
  my ($pattern) = @_;
  my @out_arr;
  open_archive();
  map { push @out_arr,$_ if /$pattern/ } $tar->list_files();
  close_archive();
  return \@out_arr;
}

sub DESTROY {
  my $self = shift;
  close_archive();
}

1;
